@@ -1,5 +1,8 @@
 #include <sys/time.h>
 
+#ifndef R_NO_REMAP
+#  define R_NO_REMAP
+#endif
 #include <R.h>
 #include <Rinternals.h>
 
